function [beta , actv_llik_max] = activity_mle(A,x,p,y,beta0)
%
% PURPOSE:  computes the MLE, log-likelihood evaluated at MLE for the
%           Autologistic model of activity in Rydberg and Shephard (2003).
%--------------------------------------------------------------------------
% USAGE:    [beta , actv_llik_max] = activity_mle(A,x,p,y,beta0)
%   where   A = (nobs x1) activity series.
%           x = (nobs x nend) a matrix each column is a function the lagged
%               values of direction (D) and size (S) series. 
%               If a constant is included, it is included in x.
%           p = (scalar) number of lags of activity series to be included.
%           y = (nobs x nexg) a matrix each column is a function of current
%               and lagged exogenous variables.
%               If there are no exogenous variables, enter [].
%           beta0 = ((nend+p+nexg) x1) vector of starting points for
%           maximisation.
%--------------------------------------------------------------------------
% RETURNS:  beta = ((nend+p+nexg) x1) vector of MLE estimates of
%                   parameters.
%                   The first nend elements of beta correspond to estimates
%                   of the coefficients of the variables in x. The next p
%                   elements correspond to estimates of the coefficients on
%                   lagged A. And the last nexg elements are estimates of
%                   the y coefficients.
%           actv_llik_max = (scalar) activity log-likelihood maximised.
%--------------------------------------------------------------------------
% NOTES:    If a constant is included as a variable, it is included in x.
%           If there are no exogenous variables, enter [] instead of y.            
%--------------------------------------------------------------------------
% OTHER FUNCTIONS USED HERE: alag.
%--------------------------------------------------------------------------
% REFERENCES: Rydberg, T. H. and N. Shephard, (2003), (Dynamics of
%             trade-by-trade price movements: Decomposition and models),
%             Journal of Financial Econometrics, 1, 2-25.
%--------------------------------------------------------------------------

% Written by Ziad Daoud: z.daoud@lse.ac.uk
% Last updated 01/10/2008

[nobsy nexg] = size(y);

% error checking
if nargin ~= 5
    error('Wrong # of arguments in activity_mle');
else
    [nobsx nend] = size(x);
    nobs = length(A);
    if nobs ~= nobsx
        error('x and A must have the same number of observations');
    end
    if (nobsy  > 0) && (nobsy ~=nobs)
        error('x, y and A must have the same number of obervations');
    end
end

model = @nested_nllik;
options = optimset('GradObj','on');
[beta , actv_nllik_min] = fminunc(model, beta0,options);
actv_llik_max = - actv_nllik_min; % this is the log-likelihood maximised


    function [nlike ngrad] = nested_nllik(beta)
    % this is a nested function to write down the -log likelihood and its
    % gradient
    
    if size(beta) == [nend+p+nexg 1]
        beta = beta;
    elseif size(beta) == [1 nend+p+nexg]
        beta = beta';
    end
    
          
    % creating the matrix of p-lagged values of A
    Alag = alag(A,p);
    
    betaX = beta(1:nend);
    betaA = beta(nend+1 : nend+p);
    
    if max(nobsy,nexg) == 0 
        % -log likelihood when there are no exogenous variables
        if p>0
            theta = x*betaX + Alag*betaA;
        elseif p==0
            theta = x*betaX;
        end
        
        temp1 = theta - log(1+exp(theta));
        temp2 = -log(1+exp(theta));
        temp3 = (A==1);
        temp4 = (A==0);
        
        llike_vec = (temp1 .* temp3) +  (temp2 .* temp4);
        llike = sum(llike_vec); %this is the log likelihood
        nlike = -llike;
        
        if nargout>1
            pt = exp(theta) ./ (1+exp(theta));
            temp5 = [x';Alag'];
            temp6 = (1 - pt) .* temp3 - (pt .* temp4);
            ngradt = -(temp5 * temp6);
            ngrad = ngradt';
        end
              
    else
        % -log likelihood with exogenous variables
        betaY = beta(nend+p+1 : nend+p+nexg);
                
        % writing down the -log likelihood
        if p>0
            theta = x*betaX + Alag*betaA + y*betaY;
        elseif p==0
            theta = x*betaX + y*betaY;
        end
        
        temp1 = theta - log(1+exp(theta));
        temp2 = -log(1+exp(theta));
        temp3 = (A==1);
        temp4 = 1-temp3;
        
        llike_vec = (temp1 .* temp3) +  (temp2 .* temp4);
        llike = sum(llike_vec); %this is the log likelihood
        nlike = -llike;
        
        if nargout >1
            pt = exp(theta) ./ (1+exp(theta));
            temp5 = [x';Alag';y'];
            temp6 = (1 - pt) .* temp3 - (pt .* temp4);
            ngradt = -(temp5 * temp6);
            ngrad = ngradt';
        end
    end
         
    end
end