function x = activity_print(result,vnames,fid)
%
% PURPOSE:  Prints the regression output for the Autologistic model of
%           Activity in Rydberg and Shephard (2003).
%--------------------------------------------------------------------------
% USAGE:    x = activity_print(result,vnames,fid)
%    where: result = a structure returned by the activity_structure function
%           vnames = ((nend + p + nexg) x1) vector of variable names 
%                    If no names are supplied, enter []. Generic names will
%                    then be used.
%           fid    = file id for printing results to a file
%                    Use 1 if you want to print results to MATLAB command
%                    window
%--------------------------------------------------------------------------
% RETURNS:  Nothing, just prints the regression results.
%--------------------------------------------------------------------------
% NOTES:  Use [] as your second argument if you don't want to supply names.
%         e.g. x = activity_print(result,[],fid)
%         You can supply names using, vnames = strvcat('constant', 'var1',
%         'sigma').
%          
%         e.g. fid = fopen('activity_output.txt','wt'); 
%         e.g. x = activity_print(result,vnames,1) if you want to print results 
%              to MATLAB command window 
%--------------------------------------------------------------------------
% OTHER FUNCTIONS USED HERE: activity_mle, alag, activity_resid, boxpierce, mprint.
%--------------------------------------------------------------------------
% REFERENCES: Rydberg, T. H. and N. Shephard, (2003), (Dynamics of
%             trade-by-trade price movements: Decomposition and models),
%             Journal of Financial Econometrics, 1, 2-25.
%--------------------------------------------------------------------------

% Written by Ziad Daoud: z.daoud@lse.ac.uk
% Last updated 02/10/2008


x = 1;
% error checking
if ~isstruct(result)
    error('activity_print requires structure argument');
elseif nargin ~= 3
    error('Wrong # of arguments to activity_print');
end


% naming variables
[vsize junk]= size(vnames);
nvar = result.nvar;

nend = result.nend;
p = result.p;
nexg = result.nexg;

if (vsize~=0) && (vsize ~= nvar) % user supplied variable names, but not the right # names
    warning('Wrong # of variable names in activity_print -- Check vnames argument')
    fprintf(fid,'Generic names are used \n \n');
    vsize =0;
elseif vsize == nvar % user supplied the right number of names
    varname = 'VARIABLE';
    varname = strvcat(varname,vnames);
end

if vsize==0 % generating generic names
    varname = 'VARIABLE';
    for index = 1:nend
        temp = ['Endogenous Variable', num2str(index)];
        varname = strvcat(varname, temp);
    end
    if p>0
        for index = nend+1 : nend+p
        temp = ['Lag A Variable', num2str(index-nend)];
        varname = strvcat(varname, temp);
        end
    end
    if nexg >0
        for index = nend+p+1 : nend+p+nexg
        temp = ['Exogenous Variable', num2str(index-nend-p)];
        varname = strvcat(varname, temp);
        end
    end
end
% naming variables is over

fprintf(fid, 'Autologistic Model for Activity \n');
fprintf(fid, '================================ \n\n');

nobtxt = ['Number of observations         = ' num2str(result.nobs) '\n'];
fprintf(fid, nobtxt);

nendtxt =['Number of endogenous variables = ' num2str(result.nend) '\n'];
fprintf(fid, nendtxt);

nptxt = ['Number of activity lags        = ' num2str(result.p) '\n'];
fprintf(fid, nptxt);

nexgtxt = ['Number of exogenous variables  = ' num2str(result.nexg) '\n\n'];
fprintf(fid, nexgtxt);

fprintf(fid, '------------------------------------------------------- \n');

temp = [result.beta result.sterror];
bstring = 'COFFECIENT';
sestring = 'STD. ERR.';
cnames = strvcat(bstring, sestring);
in.cnames = cnames;
in.rnames = varname;
in.fmt = '%16.6f';
in.fid = fid;
mprint(temp,in);

fprintf(fid, '------------------------------------------------------- \n');
lliktxt = ['Log-likelihood     = ' num2str(result.loglike) '\n'];
fprintf(fid, lliktxt);
fprintf(fid, '------------------------------------------------------- \n');

bp = result.boxpierce;

qlag = bp(:,1);

qstring = 'Q';
for index = 1:length(qlag)
    temp1 = num2str(qlag(index));
    qstring = strvcat(qstring, temp1);
end

temp = bp(:,2:3);

bpstring = 'BOX-PIERCE LAG Q';
cvstring = 'CRITICAL VALUE';
bpnames = strvcat(bpstring, cvstring);
in.cnames = bpnames;
in.rnames = qstring;
in.fmt = '%16.6f';
in.fid = fid;
mprint(temp,in);

fprintf(fid, '------------------------------------------------------- \n');



    
            
