function resid = activity_resid(A,x,p,y,beta0)
%
% PURPOSE:  Calculates the residuals for the activity model, based on the 
%           formula at the bottom of p.6 of Rydberg and Shephard (2003)
%--------------------------------------------------------------------------
% USAGE:    resid = activity_resid(A,x,p,y,beta0)
%    where: A = (nobs x1) activity series.
%           x = (nobs x nend) a matrix each column is a function the lagged
%               values of direction (D) and size (S) series. 
%               If a constant is included, it is included in x.
%           p = (scalar) number of lags of activity series to be included.
%           y = (nobs x nexg) a matrix each column is a function of current
%               and lagged exogenous variables.
%               If there are no exogenous variables, enter [].
%           beta0 = ((nend+p+nexg) x1) vector of starting points for maximisation.
%--------------------------------------------------------------------------
% RETURNS:  (nobsx1) vector of residuals.
%--------------------------------------------------------------------------
% NOTES:    If a constant is included as a variable, it is included in x.
%           If there are no exogenous variables, enter [] instead of y.            
%--------------------------------------------------------------------------
% OTHER FUNCTIONS USED HERE: activity_mle, alag
%--------------------------------------------------------------------------
% REFERENCES: Rydberg, T. H. and N. Shephard, (2003), (Dynamics of
%             trade-by-trade price movements: Decomposition and models),
%             Journal of Financial Econometrics, 1, 2-25.
%--------------------------------------------------------------------------

% Written by Ziad Daoud: z.daoud@lse.ac.uk
% Last updated 02/10/2008


[behat junk]  = activity_mle(A,x,p,y,beta0);
[junk nend] = size(x);
[nobsy nexg] = size(y);
Alag = alag(A,p);

behatX = behat(1:nend);
behatA = behat(nend+1:nend+p);

if max(nobsy,nexg) == 0   % no exogenous variables
    if p>0
        thetahat = x*behatX + Alag*behatA;
    elseif p==0
        thetahat = x*behatX;
    end
else
    behatY = behat(nend+p+1 : nend+p+nexg);
    if p>0
        thetahat = x*behatX + Alag*behatA + y*behatY;
    elseif p==0
        thetahat = x*behatX + y*behatY;
    end
end

temp1 = exp(thetahat);
pthetahat = temp1 ./ (1+temp1);

residup = A - pthetahat;
residdown = sqrt(pthetahat .* (1 - pthetahat));
resid = residup ./ residdown;


