function resid = direction_resid(D,x,p,y,beta0)
%
% PURPOSE:  Calculates the residuals for the direction model. 
%           When D(t)=0, the corresponding residual is also zero. When D(t)
%           is non-zero, the residuals are calculated based on a formula 
%           ANALOGOUS to the one at the bottom of p.6 of Rydberg 
%           and Shephard (2003)
%--------------------------------------------------------------------------
% USAGE:    resid = direction_resid(D,x,p,y,beta0)
%    where: D = (nobs x1) direction series.
%           x = (nobs x nend) a matrix each column is a function the lagged
%               values of activity (A) and size (S) series. 
%               If a constant is included, it is included in x.
%           p = (scalar) number of lags of direction series to be included.
%           y = (nobs x nexg) a matrix each column is a function of current
%               and lagged exogenous variables.
%               If there are no exogenous variables, enter [].
%           beta0 = ((nend+p+nexg) x1) vector of starting points for maximisation.
%--------------------------------------------------------------------------
% RETURNS:  (nobsx1) vector of residuals.
%--------------------------------------------------------------------------
% NOTES:    If a constant is included as a variable, it is included in x.
%           If there are no exogenous variables, enter [] instead of y.            
%--------------------------------------------------------------------------
% OTHER FUNCTIONS USED HERE: direction_mle, alag
%--------------------------------------------------------------------------
% REFERENCES: Rydberg, T. H. and N. Shephard, (2003), (Dynamics of
%             trade-by-trade price movements: Decomposition and models),
%             Journal of Financial Econometrics, 1, 2-25.
%--------------------------------------------------------------------------

% Written by Ziad Daoud: z.daoud@lse.ac.uk
% Last updated 06/10/2008


[behat junk]  = direction_mle(D,x,p,y,beta0);
[junk nend] = size(x);
[nobsy nexg] = size(y);
Dlag = alag(D,p);

behatX = behat(1:nend);
behatD = behat(nend+1:nend+p);

if max(nobsy,nexg) == 0   % no exogenous variables
    if p>0
        thetahat = x*behatX + Dlag*behatD;
    elseif p==0
        thetahat = x*behatX;
    end
else
    behatY = behat(nend+p+1 : nend+p+nexg);
    if p>0
        thetahat = x*behatX + Dlag*behatD + y*behatY;
    elseif p==0
        thetahat = x*behatX + y*behatY;
    end
end

temp1 = exp(thetahat);
temp2 = (D~=0);
pthetahat = temp1 ./ (1+temp1);

residup = D - (2*pthetahat -1);
residdown = sqrt(4*pthetahat .* (1 - pthetahat));
resid = temp2 .* (residup ./ residdown);


