function sterror = direction_sterror(D,x,p,y,beta0)
% 
% PURPOSE:  Calculates the standard errors of the MLE estimates of the
%           direction model. Output is (nend+p+nexg) x1 vector.
%           The standard errors are calculated using the BHHH estimator
%           (Greene p.481).
%--------------------------------------------------------------------------
% USAGE:    sterror = direction_sterror(A,x,p,y,beta0)
%   where   A = (nobs x1) Direction series.
%           x = (nobs x nend) a matrix each column is a function the lagged
%               values of direction (D) and size (S) series. 
%               If a constant is included, it is included in x.
%           p = (scalar) number of lags of activity series to be included.
%           y = (nobs x nexg) a matrix each column is a function of current
%               and lagged exogenous variables.
%               If there are no exogenous variables, enter [].
%           beta0 = ((nend+p+nexg) x1) vector of starting points for
%           maximisation.
%--------------------------------------------------------------------------
% RETURNS:  sterror = ((nend+p+nexg) x1) vector of standard errors of the
%                     estimates in beta.
%           The standard errors are calculated using the BHHH estimator
%           (Greene p.481).
%--------------------------------------------------------------------------
% NOTES:    If a constant is included as a variable, it is included in x.
%           If there are no exogenous variables, enter [] instead of y.            
%--------------------------------------------------------------------------
% OTHER FUNCTIONS USED HERE: alag, direction_mle
%--------------------------------------------------------------------------
% REFERENCES: Greene, W., (Econometric Analysis), 5th edition.
%             Rydberg, T. H. and N. Shephard, (2003), (Dynamics of
%             trade-by-trade price movements: Decomposition and models),
%             Journal of Financial Econometrics, 1, 2-25.
%--------------------------------------------------------------------------

% Written by Ziad Daoud: z.daoud@lse.ac.uk
% Last updated 02/10/2008


[behat junk] = direction_mle(D,x,p,y,beta0);
[junk nend] = size(x);
[nobsy nexg] = size(y);
Dlag = alag(D,p);

behatX = behat(1:nend);
behatD = behat(nend+1:nend+p);

varb = [x Dlag y];
[nobs nvarb] = size(varb);

if max(nobsy,nexg) == 0   % no exogenous variables
    if p >0
        thetahat = x*behatX + Dlag*behatD;
    elseif p==0
        thetahat = x*behatX;
    end
else
    behatY = behat(nend+p+1 : nend+p+nexg);
    if p>0
        thetahat = x*behatX + Dlag*behatD + y*behatY;
    elseif p==0
        thetahat = x*behatX + y*behatY;
    end
end

temp1 = exp(thetahat);
temp2 = 1+exp(thetahat);
temp3 = (D==1);
temp4 = (D==-1);
loderv = (1 - (temp1 ./ temp2)) .* temp3 - (temp1 ./ temp2) .* temp4;
% loderv is the derivative of the density function wrt theta
% loderv is nobs x1


% matrix of partial derivatives
gradmat = ones(nobs,nvarb)*-99;
index = 1;
while index < nobs +1
    gradmat(index,:) = loderv(index) * varb(index,:);
    index = index+1;
end

% estimating the information matrix
infmhat = gradmat' * gradmat;

% estimating the asymptotic var-covar matrix
avarhat = inv(infmhat);

% writing the vector of standard errors
avar = diag(avarhat,0);   
% returns a vector consisting of the elements of the main diagonal of
% avarhat
sterror = sqrt(avar);
