function result = direction_structure(D,x,p,y,beta0,qvec,testsize)
%
% PURPOSE:  Computes the regression for the autologistic model for dirction
%           in Rydberg and Shephard (2003).
%--------------------------------------------------------------------------
% USAGE:    result = direction_structure(D,x,p,y,beta0,qvec,alpha)
%    where: D = (nobs x1) direction series.
%           x = (nobs x nend) a matrix each column is a function the lagged
%               values of activity (A) and size (S) series. 
%               If a constant is included, it is included in x.
%           p = (scalar) number of lags of direction series to be included.
%           y = (nobs x nexg) a matrix each column is a function of current
%               and lagged exogenous variables.
%               If there are no exogenous variables, enter [].
%           beta0 = ((nend+p+nexg) x1) vector of starting points for
%                    maximisation.
%           qvec  = (nq x 1) vector of lags for which the Box-Pierce statistics
%                   is calculated.
%                   The elements of qvec must be ordered in an ascending
%                   order.
%           testsize = size of the Box-Pierce tests.
%--------------------------------------------------------------------------
% RETURNS:  a structure
%        result.beta = mle estimates of parameters     
%        result.sterror = standard error of parameters based on Hessian  
%        result.loglike = log likelihood maximised     
%        result.boxpierce = Box-Pierce statistics  
%        result.nobs = number of observations
%        result.nactv = number of non-zero direction movements
%        result.nend = number of endogenous variables included (including constant term)   
%        result.p = number of lags of D included            
%        result.nexg = number of exogenous variables included (can be 0) 
%        result.nvar = total number of variables         
%                    = nend + p + nexg           
%--------------------------------------------------------------------------
% NOTES:    If a constant is included as a variable, it is included in x.
%           If there are no exogenous variables, enter [] instead of y. 
%           The elements of qvec must be ordered in an ascending order.  
%--------------------------------------------------------------------------
% OTHER FUNCTIONS USED HERE: direction_mle, direction_sterror, alag, 
%                            direction_resid, boxpierce
%--------------------------------------------------------------------------
% REFERENCES: Rydberg, T. H. and N. Shephard, (2003), (Dynamics of
%             trade-by-trade price movements: Decomposition and models),
%             Journal of Financial Econometrics, 1, 2-25.
%--------------------------------------------------------------------------

% Written by Ziad Daoud: z.daoud@lse.ac.uk
% Last updated 06/10/2008


% error checking
if nargin ~= 7
    error('Wrong # of arguments in direction_structure');
end


result.p = p;

[nobsy nexg] = size(y);
result.nexg = nexg;
[nobsx nend] = size(x);
nobs = length(D);


result.nend = nend;

result.nobs = nobs;
result.nactv = length(find(D~=0));
result.nvar = nend + p + nexg;

[result.beta , result.loglike] = direction_mle(D,x,p,y,beta0);
result.sterror = direction_sterror(D,x,p,y,beta0);

resid = direction_resid(D,x,p,y,beta0);

[lags , bope , crval] = boxpierce(resid,qvec,testsize);
result.boxpierce = [lags , bope , crval];


