function [param , size_llik_max, sterror] = size_mle_sterror(S,x,p,y,alpha,param0)
%
% PURPOSE:  computes the MLE, log-likelihood evaluated at MLE, MLE standard
%           errors for the Negative Binomial Autoregressive model of size. 
%           This model is a simplification of the NegBin GLARMA model of
%           Rydberg and Shephard (2003).
%           The standard errors are calculated using the Hessian estimator 
%           (Greene p.481 eq.17-17)
%--------------------------------------------------------------------------
% USAGE:    [param , size_llik_max, sterror] = size_mle_sterror(S,x,p,y,alpha,param0)
%    where: S = (nobs x1) size series.
%           x = (nobs x nend) a matrix each column is a function the lagged
%               values of activity (A) and direction (D) series. 
%               If a constant is included, it is included in x.
%           p = (scalar) number of lags of S included 
%           y = (nobs x nexg) a matrix each column is a function of current
%               and lagged exogenous variables.
%               If there are no exogenous variables, enter [].
%           alpha = (scalar) value that parameter alpha takes.
%           param0 = ((nend+p+nexg) x1) vector of starting points for 
%                    maximisation.
%--------------------------------------------------------------------------
% RETURNS:  param = ((nend+p+nexg) x1) vector of MLE estimates of
%                   parameters.
%                   The first nend elements of beta correspond to estimates
%                   of the coefficients of the variables in x. The next p
%                   elements correspond to estimates of the coefficients on
%                   lagged S. And the last nexg elements are estimates of
%                   the y coefficients.
%           size_llik_max = (scalar) size log-likelihood maximised.
%           sterror = ((nend+p+nexg) x1) vector of standard errors of the
%                     estimated in param.
%                     The standard errors are calculated using the Hessian 
%                     estimator (Greene p.481 eq.17-17)
%--------------------------------------------------------------------------
% NOTES:    If a constant is included as a variable, it is included in x.
%           If there are no exogenous variables, enter [] instead of y.            
%--------------------------------------------------------------------------
% OTHER FUNCTIONS USED HERE: alag.
%--------------------------------------------------------------------------
% REFERENCES: Greene, W., (Econometric Analysis), 5th edition.
%             Rydberg, T. H. and N. Shephard, (2003), (Dynamics of
%             trade-by-trade price movements: Decomposition and models),
%             Journal of Financial Econometrics, 1, 2-25.
%--------------------------------------------------------------------------

% Written by Ziad Daoud: z.daoud@lse.ac.uk
% Last updated 06/10/2008

[nobsy nexg] = size(y);

% error checking
if nargin ~= 6
    error('Wrong # of arguments in size_mle_sterror');
else
    [nobsx nend] = size(x);
    nobs = length(S);
    if nobs ~= nobsx
        error('x and S must have the same number of observations');
    end
    if (nobsy  > 0) && (nobsy ~=nobs)
        error('x, y and S must have the same number of obervations');
    end
end

model = @nested_nllik;

[param , size_nllik_min,junk1,output,junk3,informat] = fminunc(model, param0);
size_llik_max = - size_nllik_min; % this is the log-likelihood maximised


% estimating the asymptotic var-covar matrix
avarhat = inv(informat);

% writing the vector of standard errors
avar = diag(avarhat,0);   
% returns a vector consisting of the elements of the main diagonal of
% avarhat
sterror = sqrt(avar);


    function nlike = nested_nllik(param)
    % this is a nested function to write down the -log likelihood
    
    if size(param) == [nend+p+nexg 1]
        param = param;
    elseif size(param) == [1 nend+p+nexg]
        param = param';
    end
        
    % creating the matrix of p-lagged values of A
    Slag = alag(S,p);
    
    betaX = param(1:nend);
    betaS = param(nend+1 : nend+p);
    
    if max(nobsy,nexg) == 0 
        if p>0
            thta = x*betaX + Slag*betaS;
        elseif p==0
            thta = x*betaX;
        end
    else
        betaY = param(nend+p+1 : nend+p+nexg);
        if p>0
            thta = x*betaX + Slag*betaS + y*betaY;
        elseif p==0
            thta = x*betaX + y*betaY;
        end
    end
    mu = exp(thta);
    pt = alpha ./ (mu+alpha);
    temp = (S ~=0);
    llike_vec = temp .* (alpha * log(pt) + (S-1) .* log(1-pt));
        
    llike = sum(llike_vec); %this is the log likelihood
    nlike = -llike;
    
    end
end
   