function resid = size_resid(S,x,p,y,alpha,param0)
%
% PURPOSE: Calculates the residuals for the Negative Binomial
%          Autoregressive model for size. 
%          This model is a simplification of the NegBin GLARMA model of
%          Rydberg and Shephard (2003).
%          When S(t)=0, the corresponding residual is also zero. When S(t)
%          is non-zero, the residuals are calculated based on the 
%          formula at the bottom of p.13 of Rydberg and Shephard (2003).
%--------------------------------------------------------------------------
% USAGE:    resid = size_resid(S,x,p,y,alpha,param0)
%    where: S = (nobs x1) size series.
%           x = (nobs x nend) a matrix each column is a function the lagged
%               values of activity (A) and direction (D) series. 
%               If a constant is included, it is included in x.
%           p = (scalar) number of lags of S included 
%           y = (nobs x nexg) a matrix each column is a function of current
%               and lagged exogenous variables.
%               If there are no exogenous variables, enter [].
%           alpha = (scalar) value that parameter alpha takes.
%           param0 = ((nend+p+nexg) x1) vector of starting points for 
%                    maximisation.
%--------------------------------------------------------------------------
% RETURNS:  (nobsx1) vector of residuals.
%--------------------------------------------------------------------------
% NOTES:    If a constant is included as a variable, it is included in x.
%           If there are no exogenous variables, enter [] instead of y.            
%--------------------------------------------------------------------------
% OTHER FUNCTIONS USED HERE: size_mle_sterror, alag
%--------------------------------------------------------------------------
% REFERENCES: Rydberg, T. H. and N. Shephard, (2003), (Dynamics of
%             trade-by-trade price movements: Decomposition and models),
%             Journal of Financial Econometrics, 1, 2-25.
%--------------------------------------------------------------------------

% Written by Ziad Daoud: z.daoud@lse.ac.uk
% Last updated 06/10/2008

[behat junk junk]  = size_mle_sterror(S,x,p,y,alpha,param0);
[junk nend] = size(x);
[nobsy nexg] = size(y);
Slag = alag(S,p);

behatX = behat(1:nend);
behatS = behat(nend+1:nend+p);

if max(nobsy,nexg) == 0   % no exogenous variables
    if p>0
        thetahat = x*behatX + Slag*behatS;
    elseif p==0
        thetahat = x*behatX;
    end
else
    behatY = behat(nend+p+1 : nend+p+nexg);
    if p>0
        thetahat = x*behatX + Slag*behatS + y*behatY;
    elseif p==0
         thetahat = x*behatX + y*behatY;
    end
end

muhat= exp(thetahat);
temp = (S~=0);

resid = temp .* ( (S-1-muhat) ./ sqrt(muhat + (muhat.^2 ./ alpha)));
