function result = size_structure(S,x,p,y,alpha,param0,qvec,testsize)
%
% PURPOSE:  Computes the regression for the Negative Binomial autoregressive
%           model for size.
%           This model is a simplification of the NegBin GLARMA model of
%           Rydberg and Shephard (2003).
%--------------------------------------------------------------------------
% USAGE:    resid = size_resid(S,x,p,q,y,param0,qvec,alpha)
%    where: S = (nobs x1) size series.
%           x = (nobs x nend) a matrix each column is a function the lagged
%               values of activity (A) and direction (D) series. 
%               If a constant is included, it is included in x.
%           p = (scalar) number of lags of S included 
%           y = (nobs x nexg) a matrix each column is a function of current
%               and lagged exogenous variables.
%               If there are no exogenous variables, enter [].
%           alpha = (scalar) value that parameter alpha takes.
%           param0 = ((nend+p+nexg) x1) vector of starting points for 
%                    maximisation.
%           qvec  = (nq x 1) vector of lags for which the Box-Pierce statistics
%                   is calculated.
%                   The elements of qvec must be ordered in an ascending
%                   order.
%           testsize = size of the Box-Pierce tests.
%--------------------------------------------------------------------------
% RETURNS:  a structure
%        result.beta = MLE estimates of parameters     
%        result.sterror = standard error of parameters based on Hessian  
%        result.loglike = log likelihood maximised     
%        result.boxpierce = Box-Pierce statistics  
%        result.nobs = number of observations     
%        result.nactv = number of non-zero size movements
%        result.nend = number of endogenous variables included (including constant term)   
%        result.p = number of lags of S included
%        result.nexg = number of exogenous variables included (can be 0)
%        result.nvar = number of variables 
%                    = nend + p + nexg
%--------------------------------------------------------------------------
% NOTES:    If a constant is included as a variable, it is included in x.
%           If there are no exogenous variables, enter [] instead of y. 
%           The elements of qvec must be ordered in an ascending order.  
%--------------------------------------------------------------------------
% OTHER FUNCTIONS USED HERE: size_mle_sterror, size_resid, alag, boxpierce
%--------------------------------------------------------------------------
% REFERENCES: Rydberg, T. H. and N. Shephard, (2003), (Dynamics of
%             trade-by-trade price movements: Decomposition and models),
%             Journal of Financial Econometrics, 1, 2-25.
%--------------------------------------------------------------------------

% Written by Ziad Daoud: z.daoud@lse.ac.uk
% Last updated 06/10/2008

% error checking
if nargin ~= 8
    error('Wrong # of arguments in size_structure');
end

result.p = p;

[nobsy nexg] = size(y);
[nobsx nend] = size(x);
result.nexg = nexg;
nobs = length(S);

result.nend = nend;

result.nobs = nobs;
result.nactv = length(find(S~=0));

result.nvar = p + nend +nexg;

[result.param , result.loglike, result.sterror] = size_mle_sterror(S,x,p,y,alpha,param0);
result.resid = size_resid(S,x,p,y,alpha,param0);

[lags , bope , crval] = boxpierce(result.resid,qvec,testsize);
result.boxpierce = [lags , bope , crval];

